/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "houdini/houdini.pmh"

namespace fe
{

void SurfaceAccessorHoudini::addString(void)
{
//	feLog("SurfaceAccessorHoudini::addInteger \"%s\" %p\n",
//			m_name.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
		if(m_element==SurfaceAccessibleI::e_primitive)
		{
#ifdef FE_HOUDINI_USE_GA
			m_pGdpChangeable->addStringTuple(GA_ATTRIB_PRIMITIVE,
					m_name.c_str(), 1);
#else
			m_pGdpChangeable->addPrimAttrib(m_name.c_str(),
					sizeof(UT_String),GB_ATTRIB_INDEX,NULL);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
#ifdef FE_HOUDINI_USE_GA
			m_pGdpChangeable->addStringTuple(GA_ATTRIB_POINT,
					m_name.c_str(), 1);
#else
			m_pGdpChangeable->addPointAttrib(m_name.c_str(),
					sizeof(UT_String),GB_ATTRIB_INDEX,NULL);
#endif
		}
		bindInternal(m_element,m_name);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addString"
			" could not add attribute \"%s\"\n",m_name.c_str());
		return;
	}
}
void SurfaceAccessorHoudini::addInteger(void)
{
//	feLog("SurfaceAccessorHoudini::addInteger \"%s\" %p\n",
//			m_name.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
		if(m_element==SurfaceAccessibleI::e_primitive)
		{
#ifdef FE_HOUDINI_USE_GA
			m_pGdpChangeable->addIntTuple(GA_ATTRIB_PRIMITIVE,
					m_name.c_str(), 1);
#else
			m_pGdpChangeable->addPrimAttrib(m_name.c_str(),
					sizeof(I32),GB_ATTRIB_INT,NULL);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
#ifdef FE_HOUDINI_USE_GA
			m_pGdpChangeable->addIntTuple(GA_ATTRIB_POINT,
					m_name.c_str(), 1);
#else
			m_pGdpChangeable->addPointAttrib(m_name.c_str(),
					sizeof(I32),GB_ATTRIB_INT,NULL);
#endif
		}
		bindInternal(m_element,m_name);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addInteger"
			" could not add attribute \"%s\"\n",m_name.c_str());
		return;
	}
}

void SurfaceAccessorHoudini::addReal(void)
{
//	feLog("SurfaceAccessorHoudini::addReal \"%s\" %p\n",
//			m_name.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
		if(m_element==SurfaceAccessibleI::e_primitive)
		{
#ifdef FE_HOUDINI_USE_GA
			m_pGdpChangeable->addFloatTuple(GA_ATTRIB_PRIMITIVE,
					m_name.c_str(), 1);
#else
			const Real zero=0.0;
			m_pGdpChangeable->addPrimAttrib(m_name.c_str(),
					sizeof(Real),GB_ATTRIB_FLOAT,&zero);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
#ifdef FE_HOUDINI_USE_GA
			m_pGdpChangeable->addFloatTuple(GA_ATTRIB_POINT,
					m_name.c_str(), 1);
#else
			const Real zero=0.0;
			m_pGdpChangeable->addPointAttrib(m_name.c_str(),
					sizeof(Real),GB_ATTRIB_FLOAT,&zero);
#endif
		}
		bindInternal(m_element,m_name);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addReal"
			" could not add attribute \"%s\"\n",m_name.c_str());
		return;
	}
}

void SurfaceAccessorHoudini::addSpatialVector(void)
{
//	feLog("SurfaceAccessorHoudini::addSpatialVector \"%s\" %p\n",
//			m_name.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
		if(m_element==SurfaceAccessibleI::e_primitive)
		{
#ifdef FE_HOUDINI_USE_GA
			m_pGdpChangeable->addFloatTuple(GA_ATTRIB_PRIMITIVE,
					m_name.c_str(), 3);
#else
			m_pGdpChangeable->addPrimAttrib(m_name.c_str(),
					sizeof(UT_Vector3),GB_ATTRIB_VECTOR,NULL);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
#ifdef FE_HOUDINI_USE_GA
			m_pGdpChangeable->addFloatTuple(GA_ATTRIB_POINT,
					m_name.c_str(), 3);
#else
			m_pGdpChangeable->addPointAttrib(m_name.c_str(),
					sizeof(UT_Vector3),GB_ATTRIB_VECTOR,NULL);
#endif
		}
		bindInternal(m_element,m_name);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addSpatialVector"
			" could not add attribute \"%s\"\n",m_name.c_str());
		return;
	}
}

BWORD SurfaceAccessorHoudini::bindInternal(
	SurfaceAccessibleI::Element a_element,const String& a_name)
{
	FEASSERT(m_pGdp);
	m_element=SurfaceAccessibleI::e_point;
	if(a_element>=0 && a_element<=5)
	{
		m_element=a_element;
	}
	m_name=a_name;
	m_handle.invalidate();
	m_pPrimitive=NULL;
	m_pGroup=NULL;
	m_groupIndex=0;

#ifdef FE_HOUDINI_USE_GA
	m_pIterator=NULL;
#else
	m_pPointChangeable=NULL;
	m_pPoint=NULL;
#endif

	if(m_element==SurfaceAccessibleI::e_primitiveGroup && m_name.empty())
	{
		m_element=SurfaceAccessibleI::e_primitive;
		m_attribute=SurfaceAccessibleI::e_vertices;
		return TRUE;
	}

	switch(m_element)
	{
		case SurfaceAccessibleI::e_point:
			if(m_pGdpChangeable)
			{
				m_handle=m_pGdpChangeable->getPointAttribute(a_name.c_str());
			}
			else
			{
				m_handle=m_pGdp->getPointAttribute(a_name.c_str());
			}
			break;

		case SurfaceAccessibleI::e_pointGroup:
			if(m_pGdpChangeable)
			{
				m_pGroup=m_pGdpChangeable->findPointGroup(m_name.c_str());
			}
			else
			{
#ifdef FE_HOUDINI_USE_GA
				m_pGroup=const_cast<GA_PointGroup*>(
						m_pGdp->findPointGroup(m_name.c_str()));
#else
				m_pGroup=const_cast<GB_PointGroup*>(
						m_pGdp->findPointGroup(m_name.c_str()));
#endif
			}
			break;

		case SurfaceAccessibleI::e_vertex:
			break;

		case SurfaceAccessibleI::e_primitive:
			if(m_pGdpChangeable)
			{
				m_handle=m_pGdpChangeable->getPrimAttribute(a_name.c_str());
			}
			else
			{
				m_handle=m_pGdp->getPrimAttribute(a_name.c_str());
			}
			break;

		case SurfaceAccessibleI::e_primitiveGroup:
			if(m_pGdpChangeable)
			{
				m_pGroup=m_pGdpChangeable->findPrimitiveGroup(m_name.c_str());
			}
			else
			{
#ifdef FE_HOUDINI_USE_GA
				m_pGroup=const_cast<GA_PrimitiveGroup*>(
						m_pGdp->findPrimitiveGroup(m_name.c_str()));
#else
				m_pGroup=const_cast<GB_PrimitiveGroup*>(
						m_pGdp->findPrimitiveGroup(m_name.c_str()));
#endif
			}
			break;

		case SurfaceAccessibleI::e_detail:
			break;

		default:
			;
	}
	return isBound();
}

void SurfaceAccessorHoudini::select(U32 index)
{
	FEASSERT(m_pGdp);
	switch(m_element)
	{
		case SurfaceAccessibleI::e_point:
			if(m_pGdpChangeable)
			{
#ifdef FE_HOUDINI_USE_GA
//				GEO_Point* pPointChangeable=m_pGdpChangeable->getGEOPoint(
//						m_pGdpChangeable->pointOffset(index));
//				m_handle.setElement(pPointChangeable);
				m_handle.setPoint(m_pGdpChangeable->pointOffset(index));
#else
				m_pPointChangeable=m_pGdpChangeable->points()(index);
				m_handle.setElement(m_pPointChangeable);
				m_pPoint=m_pPointChangeable;
#endif
			}
			else
			{
#ifdef FE_HOUDINI_USE_GA
//				const GEO_Point* pPoint=m_pGdp->getGEOPoint(
//						m_pGdp->pointOffset(index));
//				m_handle.setElement(pPoint);
				m_handle.setPoint(m_pGdp->pointOffset(index));
#else
				m_pPoint=m_pGdp->points()(index);
				m_handle.setElement(m_pPoint);
				m_pPointChangeable=NULL;
#endif
			}
			break;

		case SurfaceAccessibleI::e_primitive:
			if(m_pGdpChangeable)
			{
#ifdef FE_HOUDINI_USE_GA
				GEO_Primitive* pPrimitive=m_pGdpChangeable->getGEOPrimitive(
						m_pGdpChangeable->primitiveOffset(index));

//				m_handle.setPrimitive(m_pGdpChangeable->primitiveOffset(index));
#else
				GEO_Primitive* pPrimitive=
						m_pGdpChangeable->primitives()(index);
#endif
				m_pPrimitive=pPrimitive; //* to const
				m_handle.setElement(pPrimitive);
			}
			else
			{
#ifdef FE_HOUDINI_USE_GA
				m_pPrimitive=m_pGdp->getGEOPrimitive(
						m_pGdp->primitiveOffset(index));

//				m_handle.setPrimitive(m_pGdp->primitiveOffset(index));
#else
				m_pPrimitive=m_pGdp->primitives()(index);
#endif
				m_handle.setElement(m_pPrimitive);
			}
			break;

		case SurfaceAccessibleI::e_pointGroup:
		{

/* TODO for GA
	GA_PointGroup* GA_Detail::findPointGroup(char *name)
	GA_Range GA_Detail::getPointRange(const GA_PointGroup *group)
	GA_Iterator::GA_Iterator(GA_Range& range)
	GA_Offset GA_Iterator::getOffset()

	GA_Offset ptoff;
	UT_Vector3 pos;
	for( GA_Iterator it( gdp->getPointRange() ); not it.atEnd(); it.advance() )
	{
		ptoff = it.getOffset();

		pos = P_h.get( ptoff );
		pos -= avg;
		P_h.set( ptoff, pos );
*/

#ifdef FE_HOUDINI_USE_GA
			if(m_pGdpChangeable)
			{
				GA_PointGroup* pPointGroup=(GA_PointGroup*)m_pGroup;
				if(!m_pIterator)
				{
					m_groupIndex=0;
					GA_Range range=
							m_pGdpChangeable->getPointRange(pPointGroup);
					m_pIterator=new GA_Iterator(range);
				}
				U32 m=m_groupIndex;
				if(m>index)
				{
					m_pIterator->rewind();
					m=0;
				}
				for(;m<index && m_pIterator->isValid();m++)
				{
					m_pIterator->advance();
				}
				m_groupIndex=m;
			}
			else
			{
				const GA_PointGroup* pPointGroup=(const GA_PointGroup*)m_pGroup;
				if(!m_pIterator)
				{
					m_groupIndex=0;
					GA_Range range=m_pGdp->getPointRange(pPointGroup);
					m_pIterator=new GA_Iterator(range);
				}
				U32 m=m_groupIndex;
				if(m>index)
				{
					m_pIterator->rewind();
					m=0;
				}
				for(;m<index && m_pIterator->isValid();m++)
				{
					m_pIterator->advance();
				}
				m_groupIndex=m;
			}
#else
			GB_PointGroup& rPointGroup=*(GB_PointGroup*)m_pGroup;
			if(m_pGdpChangeable)
			{
				if(!m_pPointChangeable)
				{
					m_groupIndex=0;
					m_pPointChangeable=
							m_pGdpChangeable->points().head(rPointGroup);
				}
				U32 m=m_groupIndex;
				for(;m>index && m_pPointChangeable;m--)
				{
					m_pPointChangeable=
							m_pGdpChangeable->points().prev(
							m_pPointChangeable,rPointGroup);
				}
				for(;m<index && m_pPointChangeable;m++)
				{
					m_pPointChangeable=
							m_pGdpChangeable->points().next(
							m_pPointChangeable,rPointGroup);
				}
				m_groupIndex=m;
				m_pPoint=m_pPointChangeable;
			}
			else
			{
				m_pPointChangeable=NULL;

				if(!m_pPoint)
				{
					m_groupIndex=0;
					m_pPoint=m_pGdp->points().head(rPointGroup);
				}
				U32 m=m_groupIndex;
				for(;m>index && m_pPoint;m--)
				{
					m_pPoint=m_pGdp->points().prev(m_pPoint,rPointGroup);
				}
				for(;m<index && m_pPoint;m++)
				{
					m_pPoint=m_pGdp->points().next(m_pPoint,rPointGroup);
				}
				m_groupIndex=m;
			}
#endif
		}
			break;

		case SurfaceAccessibleI::e_primitiveGroup:
		{
#ifdef FE_HOUDINI_USE_GA
			GA_PrimitiveGroup* pPrimitiveGroup=(GA_PrimitiveGroup*)m_pGroup;
			if(!m_pIterator)
			{
				m_groupIndex=0;
				GA_Range range=m_pGdp->getPrimitiveRange(pPrimitiveGroup);
				m_pIterator=new GA_Iterator(range);
			}
			U32 m=m_groupIndex;
			if(m>index)
			{
				m_pIterator->rewind();
				m=0;
			}
			for(;m<index && m_pIterator->isValid();m++)
			{
				m_pIterator->advance();
			}
			m_pPrimitive=m_pGdp->getGEOPrimitive(m_pIterator->getOffset());
#else
			GB_PrimitiveGroup& rPrimitiveGroup=*(GB_PrimitiveGroup*)m_pGroup;
			if(!m_pPrimitive)
			{
				m_groupIndex=0;
				m_pPrimitive=m_pGdp->primitives().head(rPrimitiveGroup);
			}
			U32 m=m_groupIndex;
			for(;m>index && m_pPrimitive;m--)
			{
				m_pPrimitive=
						m_pGdp->primitives().prev(m_pPrimitive,rPrimitiveGroup);
			}
			for(;m<index && m_pPrimitive;m++)
			{
				m_pPrimitive=
						m_pGdp->primitives().next(m_pPrimitive,rPrimitiveGroup);
			}
#endif
			m_groupIndex=m;
		}
			break;

//		case SurfaceAccessibleI::e_vertex:
//			break;

//		case SurfaceAccessibleI::e_detail:
//			break;

		default:
			;
	}
}

} // namespace
