/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __image_ImageI_h__
#define __image_ImageI_h__

namespace fe
{

/**************************************************************************//**
    @brief Image and texture handling

	@ingroup image

	For functions without an id argument, the currently selected image
	is affected.

	Depth refers to 3D images, such as with voxels,
	Bit planes refer to the magnitude of storage per element.
*//***************************************************************************/
class FE_DL_EXPORT ImageI: virtual public Component
{
	public:
		enum	Format
				{
					e_none,
					e_colorindex,
					e_rgb,
					e_rgba
				};

virtual	U32		createSelect(void)											=0;
virtual	U32		loadSelect(String filename)									=0;
virtual U32		interpretSelect(void* data,U32 size)						=0;
virtual U32		interpretSelect(String a_source)							=0;
virtual	BWORD	save(String filename)										=0;
virtual	void	select(U32 id)												=0;
virtual	void	unload(U32 id)												=0;

virtual	void	setFormat(ImageI::Format format)							=0;
virtual	ImageI::Format	format(void) const									=0;

virtual	void	resize(U32 width,U32 height,U32 depth)						=0;
virtual	void	replaceRegion(U32 x,U32 y,U32 z,
						U32 width,U32 height,U32 depth,void* data)			=0;

virtual	U32		width(void) const											=0;
virtual	U32		height(void) const											=0;
virtual	U32		depth(void) const											=0;
virtual	void*	raw(void) const												=0;

virtual	U32		regionCount(void) const										=0;
virtual	String	regionName(U32 a_regionIndex) const							=0;
virtual	Box2i	regionBox(String a_regionName) const						=0;
virtual	String	pickRegion(I32 a_x,I32 a_y) const							=0;
};

} // namespace

#endif /* __image_ImageI_h__ */
